@@ -30,6 +30,7 @@ __all__ = [
     'TestDBusMockNoUpdate',
     'TestDBusMockUpdateAutoSuccess',
     'TestDBusMockUpdateManualSuccess',
+    'TestDBusProgress',
     'TestDBusRegressions',
     ]
 
@@ -89,6 +90,20 @@ class AutoDownloadCancelingReactor(Reactor):
         self.quit()
 
 
+class ProgressRecordingReactor(Reactor):
+    def __init__(self):
+        super().__init__(dbus.SystemBus())
+        self.react_to('UpdateDownloaded')
+        self.react_to('UpdateProgress')
+        self.progress = []
+
+    def _do_UpdateDownloaded(self, signal, path, *args, **kws):
+        self.quit()
+
+    def _do_UpdateProgress(self, signal, path, *args, **kws):
+        self.progress.append(args)
+
+
 class _TestBase(unittest.TestCase):
     """Base class for all DBus testing."""
 
@@ -214,10 +229,9 @@ class _LiveTesting(_TestBase):
             self.config.updater.cache_partition, 'reboot.log')
 
     def tearDown(self):
-        self.iface.CancelUpdate()
         # Consume the UpdateFailed that results from the cancellation.
-        reactor = SignalCapturingReactor('UpdateFailed')
-        reactor.run(timeout=15)
+        reactor = SignalCapturingReactor('TornDown')
+        reactor.run(self.iface.TearDown, timeout=15)
         safe_remove(self.config.system.build_file)
         for updater_dir in (self.config.updater.cache_partition,
                             self.config.updater.data_partition):
@@ -568,19 +582,29 @@ class TestDBusApply(_LiveTesting):
         # the empty string for this call.  But it will restart the server.
         self.assertNotEqual(self.iface.ApplyUpdate(), '')
 
+    def test_cancel_while_not_downloading(self):
+        # If we call CancelUpdate() when we're not downloading anything, no
+        # UpdateFailed signal is sent.
+        self.download_manually()
+        reactor = SignalCapturingReactor('UpdateAvailableStatus')
+        reactor.run(self.iface.CheckForUpdate)
+        self.assertEqual(len(reactor.signals), 1)
+        # Since we're downloading manually, no signal will be sent.
+        reactor = SignalCapturingReactor('UpdateFailed')
+        reactor.run(self.iface.CancelUpdate, timeout=15)
+        self.assertEqual(len(reactor.signals), 0)
+
     def test_cancel_manual(self):
         # While manually downloading, cancel the update.
         self.download_manually()
         # The downloads can be canceled when there is an update available.
         reactor = SignalCapturingReactor('UpdateAvailableStatus')
         reactor.run(self.iface.CheckForUpdate)
-        # Cancel future operations.
+        # Cancel future operations.  However, since no download is in
+        # progress, we will not get a signal.
         reactor = SignalCapturingReactor('UpdateFailed')
-        reactor.run(self.iface.CancelUpdate)
-        self.assertEqual(len(reactor.signals), 1)
-        failure_count, reason = reactor.signals[0]
-        self.assertEqual(failure_count, 1)
-        self.assertNotEqual(reason, '')
+        reactor.run(self.iface.CancelUpdate, timeout=15)
+        self.assertEqual(len(reactor.signals), 0)
         self.assertFalse(os.path.exists(self.command_file))
         # Try to download the update again, though this will fail again.
         reactor = SignalCapturingReactor('UpdateFailed')
@@ -1192,19 +1216,19 @@ class TestDBusGetSet(_TestBase):
         # auto_download requires an integer between 0 and 2.  Don't forget
         # that it gets pre-populated when the database is created.
         self.iface.SetSetting('auto_download', 'standby')
-        self.assertEqual(self.iface.GetSetting('auto_download'), '0')
+        self.assertEqual(self.iface.GetSetting('auto_download'), '1')
         self.iface.SetSetting('auto_download', '-1')
+        self.assertEqual(self.iface.GetSetting('auto_download'), '1')
+        self.iface.SetSetting('auto_download', '0')
         self.assertEqual(self.iface.GetSetting('auto_download'), '0')
-        self.iface.SetSetting('auto_download', '1')
-        self.assertEqual(self.iface.GetSetting('auto_download'), '1')
         self.iface.SetSetting('auto_download', '3')
-        self.assertEqual(self.iface.GetSetting('auto_download'), '1')
+        self.assertEqual(self.iface.GetSetting('auto_download'), '0')
         self.iface.SetSetting('auto_download', '2')
         self.assertEqual(self.iface.GetSetting('auto_download'), '2')
 
     def test_prepopulated_settings(self):
         # Some settings are pre-populated.
-        self.assertEqual(self.iface.GetSetting('auto_download'), '0')
+        self.assertEqual(self.iface.GetSetting('auto_download'), '1')
 
     def test_setting_changed_signal(self):
         reactor = SignalCapturingReactor('SettingChanged')
@@ -1219,7 +1243,7 @@ class TestDBusGetSet(_TestBase):
         self.assertEqual(len(reactor.signals), 0)
         # This is the default value, so nothing changes.
         reactor = SignalCapturingReactor('SettingChanged')
-        reactor.run(partial(self.iface.SetSetting, 'auto_download', '0'),
+        reactor.run(partial(self.iface.SetSetting, 'auto_download', '1'),
                     timeout=15)
         self.assertEqual(len(reactor.signals), 0)
         # This is a bogus value, so nothing changes.
@@ -1229,11 +1253,11 @@ class TestDBusGetSet(_TestBase):
         self.assertEqual(len(reactor.signals), 0)
         # Change back.
         reactor = SignalCapturingReactor('SettingChanged')
-        reactor.run(partial(self.iface.SetSetting, 'auto_download', '1'))
+        reactor.run(partial(self.iface.SetSetting, 'auto_download', '0'))
         self.assertEqual(len(reactor.signals), 1)
         key, new_value = reactor.signals[0]
         self.assertEqual(key, 'auto_download')
-        self.assertEqual(new_value, '1')
+        self.assertEqual(new_value, '0')
         # Change back.
         reactor = SignalCapturingReactor('SettingChanged')
         reactor.run(partial(self.iface.SetSetting, 'min_battery', '30'))
@@ -1268,3 +1292,27 @@ class TestDBusInfoNoDetails(_LiveTesting):
         self.assertEqual(channel, 'stable')
         self.assertEqual(last_update, '2022-08-01 04:45:45')
         self.assertEqual(details, {})
+
+
+class TestDBusProgress(_LiveTesting):
+    def test_progress(self):
+        self.download_always()
+        self._set_build(0)
+        reactor = SignalCapturingReactor('UpdateAvailableStatus')
+        reactor.run(self.iface.CheckForUpdate)
+        self.assertEqual(len(reactor.signals), 1)
+        # Start the download and watch the progress meters.
+        reactor = ProgressRecordingReactor()
+        reactor.schedule(self.iface.DownloadUpdate)
+        reactor.run()
+        # The only progress we can count on is the first and last ones.  All
+        # will have an eta of 0, since that value is not calculatable right
+        # now.  The first progress will have percentage 0 and the last will
+        # have percentage 100.
+        self.assertGreaterEqual(len(reactor.progress), 2)
+        percentage, eta = reactor.progress[0]
+        self.assertEqual(percentage, 0)
+        self.assertEqual(eta, 0)
+        percentage, eta = reactor.progress[-1]
+        self.assertEqual(percentage, 100)
+        self.assertEqual(eta, 0)
